import { RRule, RRuleSet, rrulestr } from "rrule";
import { getHomeUserSchedule1 } from "../../api/request";
import { getDateMap, getExcludeDate, getNextDateList, getWeekNumber, getBlankList, toLocaleDateString } from './schedule.js';
import {
  getFormatDate,
  throttle,
  padZero
} from "../../utils/utils";
const currentDate = new Date();
let minYear = 2020;
let maxYear = currentDate.getFullYear() + 1;
const DateMap = getDateMap(minYear, maxYear);
const weekList = ["周日", "周一", "周二", "周三", "周四", "周五", "周六"];
import create from "dd-store";
import { once } from "events";
create.Page({
  data: {
    scheduleList: [],
    todayStr: toLocaleDateString(currentDate),
    currentData: toLocaleDateString(currentDate),
    currentDataIndex: 0,
    show: false,
    loading: true
  },
  scrollTop: 0,
  pageSize: 10,
  totalPages: 1,
  currentPage: 1,
  firstPage: 1,
  scheduleList: [],
  getRpx() {
    dd.createSelectorQuery().select('.date').boundingClientRect().exec((rect) => {
      if (rect[0]) {
        this.otherHeight = rect[0].height;
        this.dateHeight = 128 * rect[0].height / 96
      }
    })
  },
  onLoad() {
    this.getData();
    this.$store.locationSchedule = () => {
      this.locationScheduleList();
    };
  },
  getPagination(scheduleList = []) {
    this.totalPages = Math.ceil(scheduleList.length / this.pageSize);
    for (let i = 0; i < scheduleList.length; i++) {
      if (scheduleList[i].dateStr == toLocaleDateString(currentDate)) {
        this.firstPage = Math.floor(i / this.pageSize);
        this.currentPage = this.firstPage + 2;
        break
      }
    }
  },
  getData: throttle(function (e) {
    getHomeUserSchedule1({
      startTime: "2020-01-01 00:00:00",
      endTime: "2025-12-30 23:59:59"
    }).then(res => {
      this.scheduleList = this.getScheduleList(res.data.data);
      this.getPagination(this.scheduleList);
      // 第一次加载
      this.setData({
        scheduleList: this.scheduleList.slice(this.firstPage * this.pageSize, this.currentPage * this.pageSize),
        loading: false
      });
    });
  }, 1000),
  //生成DateMap值
  setDateMapValue(startTime, endTime, item) {
    const pushItem = {
      endTime: endTime,
      startTime: startTime,
      planDate: getFormatDate(startTime, "yyyyMMdd", "-"),
      thisDayStartTime: getFormatDate(startTime, 'HH:mm'),
      thisDayEndTime: getFormatDate(endTime, 'HH:mm'),
      scheduleTemplateId: item.id,
      confirmAttendance: item.confirmAttendance,
      title: item.title,
    };
    if (DateMap.has(toLocaleDateString(startTime))) {
      DateMap.get(toLocaleDateString(startTime)).push(pushItem)
    }
    // 如果是跨天会议，需要手动在今天以后的其他天 添加会议日程  
    const nextDateList = getNextDateList(startTime, endTime);
    if (nextDateList.length > 0) {
      for (let i = 0; i < nextDateList.length; i++) {
        if (DateMap.has(nextDateList[i])) {
          DateMap.get(nextDateList[i]).push({
            ...pushItem,
            isDaySpan: true,
            isfirstDayOrEndDay: i === 0 ? "0" : i === nextDateList.length ? "2" : "1", // 0 第一天，1中间的天，2结束的天
            isFewDays: i + 1,
            duration: nextDateList.length + 1,
            isBeOverdue: currentDate.getTime() > endTime.getTime() ? true : false
          });
        }
      }
    }
  },
  // 根据接口返回数据生成填充DateMap
  setDateMapByResponse(response) {
    if (response) {
      const repeatIdList = []; //重复会议的ID
      // 处理重复会议 templateList
      response.templateList.forEach(item => {
        repeatIdList.push(item.id)
        // 生成rrules规则
        const initialTimeList = item.initialTime.split('-');
        const startTimeList = item.startTime.split(":");
        const DTSTART = `${initialTimeList[0]}${initialTimeList[1]}${initialTimeList[2]}T${startTimeList[0]}${startTimeList[1]}00Z`;
        const rule = RRule.fromString(
          `${item.recurrenceModel.recurrenceRule};DTSTART=${DTSTART}`
        );
        // 需要剔除某一天
        if (item.excludePlanDates) {
          const excludeData = getExcludeDate(item.excludePlanDates.split(","));
          rule.all().forEach(startTime => {
            //修正时间（时间有8小时时差）
            startTime.setHours(startTime.getHours() - 8);
            // 生成结束时间
            const endTime = new Date(startTime);
            endTime.setMinutes(endTime.getMinutes() + item.recurrenceModel.duration);
            // 剔除规则
            if (startTime.getTime() < excludeData.minFutureTime &&
              excludeData.excludeDateList.indexOf(getFormatDate(startTime, "yyyyMMdd", "-")) === -1) {
              this.setDateMapValue(startTime, endTime, item);
            }
          });
        }
        // 正常生成数据
        else {
          rule.all().forEach(startTime => {
            //修正时间（时间有8小时时差）
            startTime.setHours(startTime.getHours() - 8);
            // 生成结束时间
            const endTime = new Date(startTime);
            endTime.setMinutes(endTime.getMinutes() + item.recurrenceModel.duration);
            this.setDateMapValue(startTime, endTime, item);
          });
        }
      });

      // 处理单次会议 scheduleList
      response.scheduleList.forEach(item => {

        if (!item.scheduleTemplateId) {
          this.setDateMapValue(new Date(item.startTime.replace(/-/g, '/')), new Date(item.endTime.replace(/-/g, '/')), item);
        }
        // 有scheduleTemplateId则是虚拟会议转成实体会议，需要替换rrule生成的会议记录
        else if (repeatIdList.includes(item.scheduleTemplateId)) {
          const list = DateMap.get(toLocaleDateString(new Date(item.planDate.replace(/-/g, '/')))) || [];
          list.forEach((o, index) => {
            if (o.scheduleTemplateId == item.scheduleTemplateId) {
              list.splice(index, 1)
            }
          });
          this.setDateMapValue(new Date(item.startTime.replace(/-/g, '/')), new Date(item.endTime.replace(/-/g, '/')), item);
        }
      });
    }
  },
  //日程数据
  getScheduleList(response) {
    this.setDateMapByResponse(response);
    // 取出DateMap的value值平铺到DateList
    const DateList = [];
    DateMap.forEach(function (value, key) {
      const keyDate = new Date(key);
      const year = keyDate.getFullYear();
      const month = keyDate.getMonth();
      const date = keyDate.getDate();
      const day = keyDate.getDay();

      // 生成年
      if (month === 0 && date === 1) {
        DateList.push({
          type: "year",
          value: year,
          dateStr: `${year}`
        });
      }
      //生成月
      if (date === 1) {
        DateList.push({
          type: "month",
          value: month + 1,
          dateStr: `${year}/${month + 1}`
        });
      }
      //生成周
      if (day === 0) {
        const rangeDate = new Date(year, month, date + 6);
        const rangeMonth2 = new Date(rangeDate).getMonth();
        const rangeDay2 = new Date(rangeDate).getDate();
        DateList.push({
          type: "week",
          value: `第${getWeekNumber(year, month, date)}周，${month + 1}月${date}日 - ${
            rangeMonth2 == month ? "" : rangeMonth2 + 1 + "月"
            }${rangeDay2}日`,
          dateStr: `${year}/${month + 1}/${date}-week`
        });
      }
      // 有日程长度
      if (value.length !== 0) {
        value.forEach((item, index) => {
          if (index === 0) {
            DateList.push({
              type: key !== toLocaleDateString(currentDate) ? "day" : 'today',
              week: weekList[day],
              hasDateLabel: padZero(date),
              value: item,
              dateStr: `${key}`
            });
          } else {
            DateList.push({
              type: "day",
              value: item,
              dateStr: `${key}/${item.id || item.scheduleTemplateId}`
            });
          }
        });
      } else if (value.length === 0 && key === toLocaleDateString(currentDate)) {
        DateList.push({
          type: "thisday",
          value: "",
          hasDateLabel: padZero(date),
          week: weekList[day],
          dateStr: key
        });
      }
    });
    return DateList;
  },
  lower() {
    if (this.currentPage < this.totalPages) {
      this.currentPage++;
      this.setData({
        scheduleList: this.scheduleList.slice(this.firstPage * this.pageSize, this.currentPage * this.pageSize)
      });
    } else {
      //超出加载一年的数据
      maxYear++;
      this.scheduleList.push(...getBlankList(maxYear));
      this.setData({
        scheduleList: this.scheduleList.slice(this.firstPage * this.pageSize)
      })
    }
  },
  onPullDownRefresh() {
    //到顶部的时候才下拉加载数据，否则定位距离太长
    if (this.scrollTop !== 0) {
      dd.stopPullDownRefresh();
      return
    }
    const todayStr = this.data.scheduleList[0].dateStr;
    if (this.firstPage > 0) {
      this.firstPage--;
      this.setData({
        todayStr: '',
        scheduleList: this.scheduleList.slice(this.firstPage * this.pageSize, this.currentPage * this.pageSize)
      }, () => {
        this.setData({
          todayStr: todayStr
        });
        dd.stopPullDownRefresh();

      });
    } else {
      minYear--;
      const yearData = getBlankList(minYear);
      this.scheduleList.unshift(...yearData);
      this.setData({
        todayStr: '',
        scheduleList: this.scheduleList.slice(0, this.currentPage * this.pageSize),
        //加载一年的数据重定位可能会有点问题，重置一下
        currentData: todayStr,
        currentDataIndex: yearData.length
      }, () => {
        this.setData({
          todayStr: todayStr
        });
        dd.stopPullDownRefresh()
      })
    }
  },
  //计算高度
  caculateHeight(scrollTop) {
    let height = 0;
    let i = 0;
    while (height <= scrollTop) {
      if (this.data.scheduleList[i].type.indexOf('day') !== '-1') {
        height += this.dateHeight
      } else {
        height += this.otherHeight
      }
      i++
    }
    return i
    // for (let i = 0; i < this.data.scheduleList.length; i++) {
    //   if (this.data.scheduleList[i].type.indexOf('day') !== '-1') {
    //     height += this.dateHeight
    //   } else {
    //     height += this.otherHeight
    //   }
    //   if (height >= scrollTop) {
    //     index = i;
    //     break;
    //   }
    // }
    // return index
  },
  scroll(e) {
    //计算有偏差
    // const index = this.caculateHeight(e.detail.scrollTop);
    // console.log(index)
    // if (this.data.scheduleList[index].dateStr.split('/')[1] !== this.data.currentData.split('/')[1]) {
    //   console.log('setDate: ' + this.data.scheduleList[index].dateStr)
    //   this.setData({
    //     currentData: this.data.scheduleList[index].dateStr
    //   })
    // }

    //循环次数过多
    // dd.createSelectorQuery().selectAll('.li').boundingClientRect().exec((rect) => {
    //   for (let i = rect[0].length; i > 0; i--) {
    //     if (rect[0][i] && rect[0][i].top < 50) {
    //       if (this.data.scheduleList[i].dateStr.split('/')[1] !== this.data.currentData.split('/')[1]) {
    //         this.setData({
    //           currentData: this.data.scheduleList[i].dateStr,
    //         })
    //       }
    //       break;
    //     }
    //   }
    // })

    // 一次性惯性拉到顶速度跟不上，重置
    if (e.detail.scrollTop == 0) {
      this.setData({
        currentData: this.data.scheduleList[0].dateStr,
        currentDataIndex: 0
      })
    }

    //每次计算偏移值
    const offset = this.getScrollOffset(e.detail.scrollTop);
    dd.createSelectorQuery().selectAll('.li').boundingClientRect().exec((rect) => {
      if (offset == 'next') {
        for (let i = this.data.currentDataIndex; i < rect[0].length; i++) {
          if (rect[0][i].top > 0) {
            if (this.data.scheduleList[i].dateStr.split('/')[1] !== this.data.currentData.split('/')[1]) {
              this.setData({
                currentData: this.data.scheduleList[i].dateStr,
                currentDataIndex: i
              })
            }
            break;
          }
        }
      }
      else if (offset == 'before') {
        for (let i = this.data.currentDataIndex; i > 0; i--) {
          if (rect[0][i].top < 50) {
            if (this.data.scheduleList[i].dateStr.split('/')[1] !== this.data.currentData.split('/')[1]) {
              this.setData({
                currentData: this.data.scheduleList[i].dateStr,
                currentDataIndex: i
              })
            }
            break;
          }
        }
      }
    })
  },
  //判断滚动方向
  getScrollOffset(scrollTop) {
    let offset = '';
    if (scrollTop > this.scrollTop) {
      //向下滚动 
      offset = 'next'
    } else {
      //向上滚动 
      offset = 'before'
    }
    //给scrollTop重新赋值 
    this.scrollTop = scrollTop;
    return offset
  },
  nextDetail: throttle(function (e) {
    let item = e.target.dataset.item;
    dd.navigateTo({
      url: `./../meetingDetail/meetingDetail?scheduleItem=${JSON.stringify(
        item
      )}`
    });
  }, 1000),
  closePop() {
    this.setData({
      show: false
    });
  },
  changesidebar() {
    this.setData({
      show: true
    });
  },
  preventSort() {
    return false;
  },
  nextPage() {
    dd.navigateTo({ url: "./../createMeeting/createMeeting" });
  },
  locationScheduleList() {
    //需要先清空todayStr再设置才有效
    this.setData(
      {
        todayStr: ""
      },
      () => {
        this.setData({
          todayStr: toLocaleDateString(currentDate)
        });
        this.$store.data.isLocation = false;
        this.update();
      }
    );
  }
});
